@@ -23,7 +23,6 @@ import os.path
 import re
 import sys
 import traceback
-from collections import OrderedDict
 from io import BytesIO
 
 from translate import __version__
@@ -31,19 +30,17 @@ from translate.misc import progressbar
 
 
 class ProgressBar:
-    progress_types = OrderedDict(
-        [
-            ("dots", progressbar.DotsProgressBar),
-            ("none", progressbar.NoProgressBar),
-            ("bar", progressbar.HashProgressBar),
-            ("names", progressbar.MessageProgressBar),
-            ("verbose", progressbar.VerboseProgressBar),
-        ]
-    )
+    progress_types = {
+        "dots": progressbar.DotsProgressBar,
+        "none": progressbar.NoProgressBar,
+        "bar": progressbar.HashProgressBar,
+        "names": progressbar.MessageProgressBar,
+        "verbose": progressbar.VerboseProgressBar,
+    }
 
     def __init__(self, progress_type, allfiles):
         """Set up a progress bar appropriate to the progress_type and files."""
-        if progress_type in ("bar", "verbose"):
+        if progress_type in {"bar", "verbose"}:
             file_count = len(allfiles)
             self._progressbar = self.progress_types[progress_type](0, file_count)
             logger = logging.getLogger(os.path.basename(sys.argv[0])).getChild(
@@ -66,10 +63,10 @@ class ProgressBar:
 
 
 class ManPageOption(optparse.Option):
-    ACTIONS = optparse.Option.ACTIONS + ("manpage",)
+    ACTIONS = (*optparse.Option.ACTIONS, "manpage")
 
     def take_action(self, action, dest, opt, value, values, parser):
-        """take_action that can handle manpage as well as standard actions"""
+        """take_action that can handle manpage as well as standard actions."""
         if action == "manpage":
             parser.print_manpage()
             sys.exit(0)
@@ -86,19 +83,16 @@ class ManHelpFormatter(optparse.HelpFormatter):
         """Return a comma-separated list of option strings & metavariables."""
         if option.takes_value():
             metavar = option.metavar or option.dest.upper()
-            metavar = "\\fI%s\\fP" % metavar
+            metavar = f"\\fI{metavar}\\fP"
             short_opts = [sopt + metavar for sopt in option._short_opts]
             long_opts = [lopt + "\\fR=\\fP" + metavar for lopt in option._long_opts]
         else:
             short_opts = option._short_opts
             long_opts = option._long_opts
 
-        if self.short_first:
-            opts = short_opts + long_opts
-        else:
-            opts = long_opts + short_opts
+        opts = short_opts + long_opts if self.short_first else long_opts + short_opts
 
-        return "\\fB%s\\fP" % ("\\fR, \\fP".join(opts))
+        return "\\fB{}\\fP".format("\\fR, \\fP".join(opts))
 
 
 class StdoutWrapper:
@@ -124,13 +118,13 @@ class RecursiveOptionParser(optparse.OptionParser):
     def __init__(
         self, formats, usetemplates=False, allowmissingtemplate=False, description=None
     ):
-        """Construct the specialized Option Parser.
+        """
+        Construct the specialized Option Parser.
 
         :type formats: Dictionary
         :param formats: See :meth:`~.RecursiveOptionParser.setformats`
         for an explanation of the formats parameter.
         """
-
         super().__init__(version="%prog " + __version__.sver, description=description)
         self.setmanpageoption()
         self.setprogressoptions()
@@ -144,8 +138,9 @@ class RecursiveOptionParser(optparse.OptionParser):
         return os.path.basename(sys.argv[0])
 
     def setmanpageoption(self):
-        """creates a manpage option that allows the optionparser to generate a
-        manpage
+        """
+        Creates a manpage option that allows the optionparser to generate a
+        manpage.
         """
         manpageoption = ManPageOption(
             None,
@@ -158,54 +153,65 @@ class RecursiveOptionParser(optparse.OptionParser):
         self.define_option(manpageoption)
 
     def format_manpage(self):
-        """returns a formatted manpage"""
+        """Returns a formatted manpage."""
         result = []
         prog = self.get_prog_name()
-        formatprog = lambda x: x.replace("%prog", prog)
-        formatToolkit = lambda x: x.replace("%prog", "Translate Toolkit")
-        result.append('.\\" Autogenerated manpage\n')
-        result.append(
-            '.TH %s 1 "%s" "" "%s"\n'
-            % (prog, formatToolkit(self.version), formatToolkit(self.version))
-        )
-        result.append(".SH NAME\n")
-        result.append(
-            "{} \\- {}\n".format(
-                self.get_prog_name(), self.description.split("\n\n")[0]
+
+        def formatprog(x):
+            return x.replace("%prog", prog)
+
+        def formatToolkit(x):
+            return x.replace("%prog", "Translate Toolkit")
+
+        result.extend(
+            (
+                '.\\" Autogenerated manpage\n',
+                f'.TH {prog} 1 "{formatToolkit(self.version)}" "" "{formatToolkit(self.version)}"\n',
+                ".SH NAME\n",
+                "{} \\- {}\n".format(
+                    self.get_prog_name(), self.description.split("\n\n")[0]
+                ),
+                ".SH SYNOPSIS\n",
+                ".PP\n",
             )
         )
-        result.append(".SH SYNOPSIS\n")
-        result.append(".PP\n")
         usage = "\\fB%prog "
         usage += " ".join(self.getusageman(option) for option in self.option_list)
         usage += "\\fP"
-        result.append("%s\n" % formatprog(usage))
+        result.append(f"{formatprog(usage)}\n")
         description_lines = self.description.split("\n\n")[1:]
         if description_lines:
-            result.append(".SH DESCRIPTION\n")
-            result.append(
-                "\n\n".join(
-                    re.sub(r"\.\. note::", "Note:", l) for l in description_lines
+            result.extend(
+                (
+                    ".SH DESCRIPTION\n",
+                    "\n\n".join(
+                        re.sub(r"\.\. note::", "Note:", l) for l in description_lines
+                    ),
                 )
             )
         result.append(".SH OPTIONS\n")
         ManHelpFormatter().store_option_strings(self)
         result.append(".PP\n")
         for option in self.option_list:
-            result.append(".TP\n")
-            result.append("%s\n" % str(option).replace("-", r"\-"))
-            result.append("%s\n" % option.help.replace("-", r"\-"))
+            result.extend(
+                (
+                    ".TP\n",
+                    "{}\n".format(str(option).replace("-", "\\-")),
+                    "{}\n".format(option.help.replace("-", "\\-")),
+                )
+            )
         return "".join(result)
 
     def print_manpage(self, file=None):
-        """outputs a manpage for the program using the help information"""
+        """Outputs a manpage for the program using the help information."""
         if file is None:
             file = sys.stdout
         file.write(self.format_manpage())
 
     def set_usage(self, usage=None):
-        """sets the usage string - if usage not given, uses getusagestring for
-        each option
+        """
+        Sets the usage string - if usage not given, uses getusagestring for
+        each option.
         """
         if usage is None:
             self.usage = "%prog " + " ".join(
@@ -233,33 +239,34 @@ class RecursiveOptionParser(optparse.OptionParser):
                 msg += ": " + errorinfo
         logging.getLogger(self.get_prog_name()).warning(msg)
 
-    def getusagestring(self, option):
-        """returns the usage string for the given option"""
+    @staticmethod
+    def getusagestring(option):
+        """Returns the usage string for the given option."""
         optionstring = "|".join(option._short_opts + option._long_opts)
         if getattr(option, "optionalswitch", False):
-            optionstring = "[%s]" % optionstring
+            optionstring = f"[{optionstring}]"
         if option.metavar:
             optionstring += " " + option.metavar
         if getattr(option, "required", False):
             return optionstring
-        else:
-            return "[%s]" % optionstring
+        return f"[{optionstring}]"
 
-    def getusageman(self, option):
-        """returns the usage string for the given option"""
+    @staticmethod
+    def getusageman(option):
+        """Returns the usage string for the given option."""
         optionstring = "\\fR|\\fP".join(option._short_opts + option._long_opts)
         if getattr(option, "optionalswitch", False):
-            optionstring = "\\fR[\\fP%s\\fR]\\fP" % optionstring
+            optionstring = f"\\fR[\\fP{optionstring}\\fR]\\fP"
         if option.metavar:
-            optionstring += " \\fI%s\\fP" % option.metavar
+            optionstring += f" \\fI{option.metavar}\\fP"
         if getattr(option, "required", False):
             return optionstring
-        else:
-            return "\\fR[\\fP%s\\fR]\\fP" % optionstring
+        return f"\\fR[\\fP{optionstring}\\fR]\\fP"
 
     def define_option(self, option):
-        """Defines the given option, replacing an existing one of the same
-        short name if neccessary...
+        """
+        Defines the given option, replacing an existing one of the same
+        short name if necessary...
         """
         for short_opt in option._short_opts:
             if self.has_option(short_opt):
@@ -270,7 +277,8 @@ class RecursiveOptionParser(optparse.OptionParser):
         self.add_option(option)
 
     def setformats(self, formats, usetemplates):
-        """Sets the format options using the given format dictionary.
+        """
+        Sets the format options using the given format dictionary.
 
         :type formats: Dictionary or iterable
         :param formats: The dictionary *keys* should be:
@@ -285,7 +293,6 @@ class RecursiveOptionParser(optparse.OptionParser):
                         The dictionary *values* should be tuples of
                         outputformat (string) and processor method.
         """
-
         self.inputformats = []
         outputformats = []
         templateformats = []
@@ -297,7 +304,7 @@ class RecursiveOptionParser(optparse.OptionParser):
             if isinstance(formatgroup, str) or formatgroup is None:
                 formatgroup = (formatgroup,)
             if not isinstance(formatgroup, tuple):
-                raise ValueError("formatgroups must be tuples or None/str/unicode")
+                raise TypeError("formatgroups must be tuples or None/str/unicode")
             if len(formatgroup) < 1 or len(formatgroup) > 2:
                 raise ValueError("formatgroups must be tuples of length 1 or 2")
             if len(formatgroup) == 1:
@@ -312,7 +319,7 @@ class RecursiveOptionParser(optparse.OptionParser):
                 outputformats.append(outputformat)
             if templateformat not in templateformats:
                 templateformats.append(templateformat)
-            self.outputoptions[(inputformat, templateformat)] = (
+            self.outputoptions[inputformat, templateformat] = (
                 outputformat,
                 processor,
             )
@@ -321,9 +328,10 @@ class RecursiveOptionParser(optparse.OptionParser):
             "-i",
             "--input",
             dest="input",
+            action="append",
             default=None,
             metavar="INPUT",
-            help="read from INPUT in %s" % (inputformathelp),
+            help=f"read from INPUT in {inputformathelp}",
         )
         inputoption.optionalswitch = True
         inputoption.required = True
@@ -346,7 +354,7 @@ class RecursiveOptionParser(optparse.OptionParser):
             dest="output",
             default=None,
             metavar="OUTPUT",
-            help="write to OUTPUT in %s" % (outputformathelp),
+            help=f"write to OUTPUT in {outputformathelp}",
         )
         outputoption.optionalswitch = True
         outputoption.required = True
@@ -360,7 +368,7 @@ class RecursiveOptionParser(optparse.OptionParser):
                 dest="template",
                 default=None,
                 metavar="TEMPLATE",
-                help="read from TEMPLATE in %s" % (templateformathelp),
+                help=f"read from TEMPLATE in {templateformathelp}",
             )
             self.define_option(templateoption)
 
@@ -373,7 +381,7 @@ class RecursiveOptionParser(optparse.OptionParser):
             default="bar",
             choices=list(ProgressBar.progress_types.keys()),
             metavar="PROGRESS",
-            help="show progress as: %s" % (", ".join(ProgressBar.progress_types)),
+            help="show progress as: {}".format(", ".join(ProgressBar.progress_types)),
         )
         self.define_option(progressoption)
 
@@ -387,31 +395,32 @@ class RecursiveOptionParser(optparse.OptionParser):
             default="message",
             choices=self.errorleveltypes,
             metavar="ERRORLEVEL",
-            help="show errorlevel as: %s" % (", ".join(self.errorleveltypes)),
+            help="show errorlevel as: {}".format(", ".join(self.errorleveltypes)),
         )
         self.define_option(errorleveloption)
 
-    def getformathelp(self, formats):
+    @staticmethod
+    def getformathelp(formats):
         """Make a nice help string for describing formats..."""
         formats = sorted(f for f in formats if f is not None)
         if len(formats) == 0:
             return ""
-        elif len(formats) == 1:
-            return "%s format" % (", ".join(formats))
-        else:
-            return "%s formats" % (", ".join(formats))
+        if len(formats) == 1:
+            return "{} format".format(", ".join(formats))
+        return "{} formats".format(", ".join(formats))
 
-    def isrecursive(self, fileoption, filepurpose="input"):
+    @staticmethod
+    def isrecursive(fileoption, filepurpose="input"):
         """Checks if fileoption is a recursive file."""
         if fileoption is None:
             return False
-        elif isinstance(fileoption, list):
+        if isinstance(fileoption, list):
             return True
-        else:
-            return os.path.isdir(fileoption)
+        return os.path.isdir(fileoption)
 
     def parse_args(self, args=None, values=None):
-        """Parses the command line options, handling implicit input/output
+        """
+        Parses the command line options, handling implicit input/output
         args.
         """
         (options, args) = super().parse_args(args, values)
@@ -452,16 +461,16 @@ class RecursiveOptionParser(optparse.OptionParser):
     def getoutputoptions(self, options, inputpath, templatepath):
         """Works out which output format and processor method to use..."""
         if inputpath:
-            inputbase, inputext = self.splitinputext(inputpath)
+            _inputbase, inputext = self.splitinputext(inputpath)
         else:
             inputext = None
         if templatepath:
-            templatebase, templateext = self.splittemplateext(templatepath)
+            _templatebase, templateext = self.splittemplateext(templatepath)
         else:
             templateext = None
         if (inputext, templateext) in self.outputoptions:
             return self.outputoptions[inputext, templateext]
-        elif (inputext, "*") in self.outputoptions:
+        if (inputext, "*") in self.outputoptions:
             outputformat, fileprocessor = self.outputoptions[inputext, "*"]
         elif ("*", templateext) in self.outputoptions:
             outputformat, fileprocessor = self.outputoptions["*", templateext]
@@ -481,21 +490,17 @@ class RecursiveOptionParser(optparse.OptionParser):
                     raise ValueError(
                         "don't know what to do with input format (no file extension), no template file"
                     )
-                elif templateext is None:
+                if templateext is None:
                     raise ValueError(
                         "don't know what to do with input format %s, no template file"
                         % (os.extsep + inputext)
                     )
-                else:
-                    raise ValueError(
-                        "don't know what to do with input format %s, template format %s"
-                        % (os.extsep + inputext, os.extsep + templateext)
-                    )
-            else:
                 raise ValueError(
-                    "don't know what to do with input format %s"
-                    % (os.extsep + inputext)
+                    f"don't know what to do with input format {os.extsep + inputext}, template format {os.extsep + templateext}"
                 )
+            raise ValueError(
+                "don't know what to do with input format %s" % (os.extsep + inputext)
+            )
         if outputformat == "*":
             if inputext:
                 outputformat = inputext
@@ -508,40 +513,39 @@ class RecursiveOptionParser(optparse.OptionParser):
                     raise ValueError(
                         "don't know what to do with input format (no file extension), no template file"
                     )
-                else:
-                    raise ValueError(
-                        "don't know what to do with input format (no file extension)"
-                    )
+                raise ValueError(
+                    "don't know what to do with input format (no file extension)"
+                )
         return outputformat, fileprocessor
 
-    def getfullinputpath(self, options, inputpath):
+    @staticmethod
+    def getfullinputpath(options, inputpath):
         """Gets the full path to an input file."""
         if options.input:
             return os.path.join(options.input, inputpath)
-        else:
-            return inputpath
+        return inputpath
 
-    def getfulloutputpath(self, options, outputpath):
+    @staticmethod
+    def getfulloutputpath(options, outputpath):
         """Gets the full path to an output file."""
         if options.recursiveoutput and options.output:
             return os.path.join(options.output, outputpath)
-        else:
-            return outputpath
+        return outputpath
 
     def getfulltemplatepath(self, options, templatepath):
         """Gets the full path to a template file."""
         if not options.recursivetemplate:
             return templatepath
-        elif templatepath is not None and self.usetemplates and options.template:
+        if templatepath is not None and self.usetemplates and options.template:
             return os.path.join(options.template, templatepath)
-        else:
-            return None
+        return None
 
     def run(self):
-        """Parses the arguments, and runs recursiveprocess with the resulting
+        """
+        Parses the arguments, and runs recursiveprocess with the resulting
         options...
         """
-        (options, args) = self.parse_args()
+        (options, _args) = self.parse_args()
         self.recursiveprocess(options)
 
     def recursiveprocess(self, options):
@@ -554,12 +558,11 @@ class RecursiveOptionParser(optparse.OptionParser):
                 inputfiles = self.recurseinputfilelist(options)
             else:
                 inputfiles = self.recurseinputfiles(options)
+        elif options.input:
+            inputfiles = [os.path.basename(options.input)]
+            options.input = os.path.dirname(options.input)
         else:
-            if options.input:
-                inputfiles = [os.path.basename(options.input)]
-                options.input = os.path.dirname(options.input)
-            else:
-                inputfiles = [options.input]
+            inputfiles = [options.input]
         options.recursiveoutput = self.isrecursive(
             options.output, "output"
         ) and getattr(options, "allowrecursiveoutput", True)
@@ -597,7 +600,7 @@ class RecursiveOptionParser(optparse.OptionParser):
                     self.checkoutputsubdir(options, os.path.dirname(outputpath))
             except Exception:
                 self.warning(
-                    "Couldn't handle input file %s" % inputpath, options, sys.exc_info()
+                    f"Couldn't handle input file {inputpath}", options, sys.exc_info()
                 )
                 continue
             try:
@@ -610,14 +613,12 @@ class RecursiveOptionParser(optparse.OptionParser):
                 )
             except Exception:
                 self.warning(
-                    "Error processing: input %s, output %s, template %s"
-                    % (fullinputpath, fulloutputpath, fulltemplatepath),
+                    f"Error processing: input {fullinputpath}, output {fulloutputpath}, template {fulltemplatepath}",
                     options,
                     sys.exc_info(),
                 )
                 success = False
             progress_bar.report_progress(inputpath, success)
-        del progress_bar
 
     def ensurerecursiveoutputdirexists(self, options):
         if not self.isrecursive(options.output, "output"):
@@ -633,19 +634,22 @@ class RecursiveOptionParser(optparse.OptionParser):
                     )
                 )
 
-    def openinputfile(self, options, fullinputpath):
+    @staticmethod
+    def openinputfile(options, fullinputpath):
         """Opens the input file."""
         if fullinputpath is None:
             return sys.stdin
         return open(fullinputpath, "rb")
 
-    def openoutputfile(self, options, fulloutputpath):
+    @staticmethod
+    def openoutputfile(options, fulloutputpath):
         """Opens the output file."""
         if fulloutputpath is None:
             return StdoutWrapper()
         return open(fulloutputpath, "wb")
 
-    def opentempoutputfile(self, options, fulloutputpath):
+    @staticmethod
+    def opentempoutputfile(options, fulloutputpath):
         """Opens a temporary output file."""
         return BytesIO()
 
@@ -662,8 +666,7 @@ class RecursiveOptionParser(optparse.OptionParser):
         if fulltemplatepath is not None:
             if os.path.isfile(fulltemplatepath):
                 return open(fulltemplatepath, "rb")
-            else:
-                self.warning("missing template file %s" % fulltemplatepath)
+            self.warning(f"missing template file {fulltemplatepath}")
         return None
 
     def processfile(
@@ -671,7 +674,7 @@ class RecursiveOptionParser(optparse.OptionParser):
     ):
         """Process an individual file."""
         inputfile = self.openinputfile(options, fullinputpath)
-        if fulloutputpath and fulloutputpath in (fullinputpath, fulltemplatepath):
+        if fulloutputpath and fulloutputpath in {fullinputpath, fulltemplatepath}:
             outputfile = self.opentempoutputfile(options, fulloutputpath)
             tempoutput = True
         else:
@@ -679,24 +682,30 @@ class RecursiveOptionParser(optparse.OptionParser):
             tempoutput = False
         templatefile = self.opentemplatefile(options, fulltemplatepath)
         passthroughoptions = self.getpassthroughoptions(options)
-        if fileprocessor(inputfile, outputfile, templatefile, **passthroughoptions):
+        result = fileprocessor(
+            inputfile, outputfile, templatefile, **passthroughoptions
+        )
+        if fullinputpath is not None:
+            inputfile.close()
+        if result:
             if tempoutput:
                 self.warning("writing to temporary output...")
                 self.finalizetempoutputfile(options, outputfile, fulloutputpath)
-            return True
-        else:
-            # remove the file if it is a file (could be stdout etc)
             if fulloutputpath and os.path.isfile(fulloutputpath):
                 outputfile.close()
-                os.unlink(fulloutputpath)
-            return False
+            return True
+        # remove the file if it is a file (could be stdout etc)
+        if fulloutputpath and os.path.isfile(fulloutputpath):
+            outputfile.close()
+            os.unlink(fulloutputpath)
+        return False
 
-    def mkdir(self, parent, subdir):
-        """Makes a subdirectory (recursively if neccessary)."""
+    @staticmethod
+    def mkdir(parent, subdir):
+        """Makes a subdirectory (recursively if necessary)."""
         if not os.path.isdir(parent):
             raise ValueError(
-                "cannot make child directory %r if parent %r does not exist"
-                % (subdir, parent)
+                f"cannot make child directory {subdir!r} if parent {parent!r} does not exist"
             )
         currentpath = parent
         subparts = subdir.split(os.sep)
@@ -706,14 +715,16 @@ class RecursiveOptionParser(optparse.OptionParser):
                 os.mkdir(currentpath)
 
     def checkoutputsubdir(self, options, subdir):
-        """Checks to see if subdir under options.output needs to be created,
-        creates if neccessary.
+        """
+        Checks to see if subdir under options.output needs to be created,
+        creates if necessary.
         """
         fullpath = os.path.join(options.output, subdir)
         if not os.path.isdir(fullpath):
             self.mkdir(options.output, subdir)
 
-    def isexcluded(self, options, inputpath):
+    @staticmethod
+    def isexcluded(options, inputpath):
         """Checks if this path has been excluded."""
         basename = os.path.basename(inputpath)
         for excludename in options.exclude:
@@ -765,8 +776,10 @@ class RecursiveOptionParser(optparse.OptionParser):
             dirstack.extend(dirs)
         return inputfiles
 
-    def splitext(self, pathname):
-        """Splits *pathname* into name and ext, and removes the extsep.
+    @staticmethod
+    def splitext(pathname):
+        """
+        Splits *pathname* into name and ext, and removes the extsep.
 
         :param pathname: A file path
         :type pathname: string
@@ -806,7 +819,7 @@ class RecursiveOptionParser(optparse.OptionParser):
                             return templatepath
             if "*" in self.inputformats:
                 for inputext1, templateext1 in self.outputoptions:
-                    if (inputext == inputext1) or (inputext1 == "*"):
+                    if inputext1 in {inputext, "*"}:
                         if templateext1 == "*":
                             templatepath = inputname
                             if self.templateexists(options, templatepath):
@@ -821,7 +834,7 @@ class RecursiveOptionParser(optparse.OptionParser):
         """Gets an output filename based on the input filename."""
         if not inputname or not options.recursiveoutput:
             return options.output
-        inputbase, inputext = self.splitinputext(inputname)
+        inputbase, _inputext = self.splitinputext(inputname)
         outputname = inputbase
         if outputformat:
             outputname += os.extsep + outputformat
@@ -829,5 +842,5 @@ class RecursiveOptionParser(optparse.OptionParser):
 
     def isvalidinputname(self, inputname):
         """Checks if this is a valid input filename."""
-        inputbase, inputext = self.splitinputext(inputname)
+        _inputbase, inputext = self.splitinputext(inputname)
         return (inputext in self.inputformats) or ("*" in self.inputformats)
